c-----------------------------------------------------------------------
      module mod_progver
c-----------------------------------------------------------------------
c --- Identify version and level of ABFLARE

c Program Versions
      character*12  :: mmodel,mversion,mlevel

c      data mversion/'1.0 '/,mlevel/'140415'/
c Updated uemin exit pseudo velocity from 0.1 to 0.00001 m/s 
c  made uemin a variable instead of a parameter
c      data mversion/'1.0 '/,mlevel/'160531'/
c updated the equations of state
c updated the flare assist to include hp fuel
c fixes to  ensure compatability with AERflare
      data mmodel/'ABFLARE'/, mversion/'2.01'/,mlevel/'220131'/
      

      end module mod_progver
      

c-----------------------------------------------------------------------
      module mod_stddefs
c-----------------------------------------------------------------------
      integer :: debug ! 0 no debug; 1=debug				

      parameter (mxvsrc=32)
      parameter (mxchem = 10 )  ! maximum number of chemicals emitted in CALPUFF file
      parameter (mxpuff=10000)
      parameter (mxkd=8)
      parameter (mxmet=44000)
      parameter (mxprop = 35 )
      parameter (mxcomp = 38 ) 
      parameter (mxpath = 255)

      parameter (fUPPER=0)
      parameter (fLOWER=1)
      
c      real   :: pi      ! uh... pi
c      real   :: ru      ! Universal Gas Constant (Ru),(kPam)/(Kkmol)
c      real   :: rk0     ! Absolute zero, reference C
c      real   :: grav    ! Gravitational Constant (g),m/s2
c      real   :: lapse   ! atmospheric default lapse rate
c      real   :: rk13    ! one third 
c      real   :: rk23    ! two thirds
c	   real   :: rmxfrel ! maximum permitted fraction of volume released
c	   real   :: tch2o   ! h2o critical temp, K
c      real   :: pch2o   ! h2o critical pressure
c      real   :: rhoch2o ! ch2o critical density
c      real   :: uemin   ! minimum stack top exit velocity
      parameter (pi       = 3.14159265358979323846D0 )
      parameter (ru       = 8.31446261815324D0   )   ! updated 200311 (https://en.wikipedia.org/wiki/Gas_constant)
      parameter (rk0      = 273.15D0   )
      parameter (grav     = 9.80665D0  )
      parameter (rk13     = 1.D0/3.D0 )
      parameter (rk23     = 2.D0/3.D0 )
	parameter (rmxfrel = 0.999D0)	! maximum permitted fraction of volume released
	parameter (tch2o = 647.096D0) ! critical temperature of water, K
	parameter (pch2o = 22064.D0)  ! criticl pressure of water, kPa
	parameter (rhoch2o = 322.D0)  ! critical density
	parameter (tcair = 132.63D0) ! critical temperature of air, K
	parameter (pcair = 3785.8D0)  ! criticl pressure of air, kPa
	parameter (rhocair = 0.D0)    ! critical density
c v141119 changed from a parameter to variable
c v141119 Changed from 0.1 to 0.00001 m/s
c to avoid high pseudo temperatures
c        parameter (uemin=0.1D0) ! m/s
c        parameter (uemin=0.00001D0) ! m/s
        
      end module mod_stddefs

      
c-----------------------------------------------------------------------
      module mod_globals
c-----------------------------------------------------------------------
      use mod_stddefs 
	real(8)   :: effminERCB ! Minimum Conversion and Combustion Efficiency ERCB
      real(8)   :: effmin  ! Minimum Conversion and Combustion Efficiency
      real(8)   :: radloss ! flame radiation loss, %
      real(8)   :: tgas    ! default gas temperature, K 
      real(8)   :: tamb    ! default ambient temperature, K
      real(8)   :: tradbkg ! Background Thermal Radiation, kW/m2
      real(8)   :: tradtot ! Maximum Recommended Total Thermal Radiation, kW/m2
      real(8)   :: u10min  ! Minimum Wind Speed at 10 m reference height, m/s
      real(8)   :: ustabf  ! Stability Class F Plume Rise Criterion Wind Speed
      real(8)   :: tgrad   ! Stability Class F Vertical Potential Temperature Gradient
      real(8)   :: slopemx ! Maximum Slope Criterion, %
      real(8)   :: uofaa   ! UofA Flare Research Project constant A, (MJ/kg)^3
      real(8)   :: uofab   ! UofA Flare Research Project constant B
      real(8)   :: tref    ! reference Temperature,K
      real(8)   :: pref    ! reference Pressure,kPa
      real(8)   :: spv     ! Specific Volume at reference conditions,m/kmol
      real(8)   :: rkm     ! 
      real(8)   :: rlapse  ! [K/m] standard lapse rate (troposphere)
      real(8)   :: rk1     ! Heat Release Rate Constant (K1),(caldm)/(s10mMJ)
      real(8)   :: rk2     ! SO2 Emission Rate Constant (K2),(gd)/(s10m%H2S)
      real(8)   :: rk4     ! Heat Release Rate Constant (K4),(Jdm)/(s10mMJ)
      real(8)   :: nhvrec  ! net heating value required, MJ/m (15C,101.325 kPa) 
      real(8)   :: nhvmin  ! net heating value mininum, MJ/m (15C,101.325 kPa)
      real(8)   :: uemin   ! minimum pseudo exit velocity, m/s
      real(8)   :: wsmin   ! minimum wind speed at anemomenter reference height, m/s
      real(8)   :: wsmax   ! maximum wind speed at anemomenter reference height, m/s

      real(8)  :: plocal  ! local pressure, kPa
      

c Property definitions      
      parameter(jph2o=1)
      parameter(jph2=2)
      parameter(jphe=3)
      parameter(jpn2=4)
      parameter(jpco2=5)
      parameter(jph2s=6)
      parameter(jpch4=7)
      parameter(jpc2h6=8)
      parameter(jpc3h8=9)
      parameter(jpic4h10=10)
      parameter(jpnc4h10=11)
      parameter(jpic5h12=12)
      parameter(jpnc5h12=13)
      parameter(jpnc6h14=14)
      parameter(jpc7=15)
      parameter(jpc8=16)
      parameter(jpc9=17)
      parameter(jpc10=18)
      parameter(jpc2h2=19)
      parameter(jpc2h4=20)
      parameter(jpc3h6=21)
      parameter(jpc4h8=22)
      parameter(jpc5h10=23)
      parameter(jpc6h12=24)
      parameter(jpco=25)
      parameter(jpnh3=26)
      parameter(jpar=27)
      parameter(jpo2=28)
      parameter(jpne=29)
      parameter(jpkr=30)
      parameter(jpn2o=31)
      parameter(jpxe=32)
      parameter(jpso2=33)
      parameter(jps=34)
      parameter(jpcos=35)
      parameter(jpcs2=36)
      parameter(jpcl2=37)
      parameter(jpair=38)


      parameter(ipmmass=1)
      parameter(ipnhv=2)
      parameter(ipghv=3)
      parameter(ipcpmass=4)
      parameter(ipcpmole=5)
      parameter(ipair=6)
      parameter(ipstoico2=7)
      parameter(ipstoicc=8)
      parameter(ipstoicp=9)
      parameter(ipstoich2o=10)
      parameter(iplfl=11)
      parameter(ipcomb=12)
      parameter(ipautoig=13)
      parameter(ipa1=14)
      parameter(ipa2=15)
      parameter(ipa3=16)
      parameter(ipa4=17)
      parameter(ipa5=18)
      parameter(ipa6=19)
      parameter(ipa7=20)
      parameter(ipab1=21)
      parameter(ipab2=22)
      parameter(ipb1=23)
      parameter(ipb2=24)
      parameter(ipb3=25)
      parameter(ipb4=26)
      parameter(ipb5=27)
      parameter(ipb6=28)
      parameter(ipb7=29)
      parameter(ipbb1=30)
      parameter(ipbb2=31)
      parameter(ipcalc=32)
      parameter(iph=33)
      parameter(ipexh=34)
      parameter(ipinert=35)

      parameter (mxnasa = (ipbb2-ipa1+1) ) ! number of NASA GLAN coefficients 9+9=18
      real(8)  :: prop(mxprop,mxcomp)

c----------------------------------------------------------------------
c
c See PROPERTIES.xlsx
c      Note a6,a7 (or a1*), b6,b7 (or b1*)
c      are provided in the properties list from the spreadsheet, but are not 
c      used.  The a6 and a1* properties are calculated
c      within this code.
c         
c  ref: references conditions at 15 C
c  -Gas Producers Suppliers Association (GPSA), 2012, Engineering Data Book, 13th Edition, SI, Vol 1 & 2 
c  -Gordon,S.,  McBride, B.J., 1971, Computer Program for Calculation of Complex Chemical Equilibrium Composition, 
c  Rocket Performance, Incident and Reflected Shocks and Chapman-Jouguet Detonations, NASA SP-273.
c  -Burcat,A., Ruscic,B., 2005, Third Millennium Ideal Gas and Condensed Phase Thermochecmical Database for 
c  Combustion with Updates from Active Thermochemical Tables.  HTTP://www.osti.gov.bridge.
c  -Zabetakis,M.G.. 1965. Flammability Characteristics of Combustible Gases and Vapors. U.S. Dept. 
c  of the Interior, Bureau of Mines, Bulletin 627."									
c         
c         
c Updates:
c     1311029: Updated with addition of a6          
c     121029: Updated to GPSA 13th Ed
c
c----------------------------------------------------------------------
c  Initialize chemo-phsyical constants
c
c   see stddefs.h for i,j constants
c
c   i=1,mxprop
c  Properties from GPSA
c     1:  ipmmass:  kg/kmol:  "Molar Mass"
c     2:  ipnhv:  "MJ/m (15C,101.325 kPa)":  "Net Heating Value"
c     3:  ipghv:  "MJ/m (15C,101.325 kPa)":  "Gross Heating Value"
c     4:  ipcpmass:  "kJ/(kgK) (15C,101.325 kPa)":  "Mass Specific Heat Cp"
c     5:  ipcpmole:  "kJ/(kmolK) (15C,101.325 kPa)":  "Molar Specific Heat Cp"
c  Combustion Properties
c     6:  ipstoico2:  O2 required:  Stoichiometric Combustion (moles/mole fuel)
c     7:  ipstoicc:  Carbons:  
c     8:  ipstoicp:  Products:  
c     9:  ipstoich2o:  H2O:  
c     10:  iplfl:  %:  Lower Flammable Limit
c     11:  ipcomb:  :  Combustible Fraction
c     12:  ipautoig :  Auto ignition Temperatures (C )
c     35:  ipair:    Air Composition
c     13:  ipa1:  a1:  Table D.1 Temperature-dependent Molar Heat Capacities of Gases at Zero Pressure (Low Temp 200-1000K)
c     14:  ipa2:  a2:  NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c     15:  ipa3:  a3:  
c     16:  ipa4:  a4:  
c     17:  ipa5:  a5:  
c     18:  ipa6:  a6:  
c     19:  ipa7:  a7:  
c     20:  ipab1: ab1:  
c     21:  ipab2: ab2:  
c     22:  ipb1:  b1:  Table D.1 Temperature-dependent Molar Heat Capacities of Gases at Zero Pressure (Hi Temp 1000-6000K)
c     23:  ipb2:  b2:  NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c     24:  ipb3:  b3:  
c     25:  ipb4:  b4:  
c     26:  ipb5:  b5:  
c     27:  ipb6:  b6:  
c     28:  ipb7:  b7:  
c     29:  ipbb1: bb1:  
c     30:  ipbb2: bb2:  
c     31:  ipcalc:   flag for components used in calculats
c     32:  iph:      flag for hydrocarbons with H atoms
c     33:  ipexh:    flag for exhaust products
c     34:  ipinert:  flag for inerts
      
      data  (prop(i,jph2o),i=1,mxprop)/1.8015300000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.8620943326D+00,
     & 3.3546188029D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -3.9479608300D+04,
     & 5.7557310200D+02,
     & 9.3178265300D-01,
     & 7.2227128600D-03,
     & -7.3425573700D-06,
     & 4.9550434900D-09,
     & -1.3369332460D-12,
     & -3.3039743100D+04,
     & 1.7242057750D+01,
     & 1.0349720960D+06,
     & -2.4126985620D+03,
     & 4.6461107800D+00,
     & 2.2919983070D-03,
     & -6.8368304800D-07,
     & 9.4264689300D-11,
     & -4.8223805300D-15,
     & -1.3842865090D+04,
     & -7.9781485100D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & -9.9990000000D+03/
      data  (prop(i,jph2),i=1,mxprop)/2.0158800000D+00,
     & 1.0223000000D+01,
     & 1.2102000000D+01,
     & 1.4267175751D+01,
     & 2.8760914254D+01,
     & 5.0000000000D-07,
     & 5.0000000000D-01,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & 4.0000000000D+00,
     & 1.0000000000D+00,
     & 4.0000000000D+02,
     & 4.0783232100D+04,
     & -8.0091860400D+02,
     & 8.2147020100D+00,
     & -1.2697144570D-02,
     & 1.7536050760D-05,
     & -1.2028602700D-08,
     & 3.3680934900D-12,
     & 2.6824846650D+03,
     & -3.0437888440D+01,
     & 5.6081280100D+05,
     & -8.3715047400D+02,
     & 2.9753645320D+00,
     & 1.2522491240D-03,
     & -3.7407161900D-07,
     & 5.9366252000D-11,
     & -3.6069941000D-15,
     & 5.3398244100D+03,
     & -2.2027747690D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & 1.0000000000D+00/
      data  (prop(i,jphe),i=1,mxprop)/4.0026000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 5.1931635800D+00,
     & 2.0786156545D+01,
     & 5.2000000000D-06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 9.2872397400D-01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 9.2872397400D-01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & 1.0000000000D+00/
      data  (prop(i,jpn2),i=1,mxprop)/2.8013480000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0395059054D+00,
     & 2.9120177890D+01,
     & 7.8085090000D-01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.2103714970D+04,
     & -3.8184618200D+02,
     & 6.0827383600D+00,
     & -8.5309144100D-03,
     & 1.3846461890D-05,
     & -9.6257936200D-09,
     & 2.5197058090D-12,
     & 7.1084608600D+02,
     & -1.0760037440D+01,
     & 5.8771240600D+05,
     & -2.2392490730D+03,
     & 6.0669492200D+00,
     & -6.1396855000D-04,
     & 1.4918066790D-07,
     & -1.9231054850D-11,
     & 1.0619543860D-15,
     & 1.2832104150D+04,
     & -1.5866400270D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpco2),i=1,mxprop)/4.4009500000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 8.3326870992D-01,
     & 3.6671739289D+01,
     & 4.0000000000D-04,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 4.9436505400D+04,
     & -6.2641160100D+02,
     & 5.3017252400D+00,
     & 2.5038138160D-03,
     & -2.1273087280D-07,
     & -7.6899887800D-10,
     & 2.8496778010D-13,
     & -4.5281984600D+04,
     & -7.0482794400D+00,
     & 1.1769624190D+05,
     & -1.7887914770D+03,
     & 8.2915231900D+00,
     & -9.2231567800D-05,
     & 4.8636768800D-09,
     & -1.8910533120D-12,
     & 6.3300365900D-16,
     & -3.9083505900D+04,
     & -2.6526692810D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jph2s),i=1,mxprop)/3.4080900000D+01,
     & 2.1905000000D+01,
     & 2.3784000000D+01,
     & 1.0016747358D+00,
     & 3.4137976503D+01,
     & 0.0000000000D+00,
     & 1.5000000000D+00,
     & 0.0000000000D+00,
     & 2.0000000000D+00,
     & 1.0000000000D+00,
     & 4.3000000000D+00,
     & 1.0000000000D+00,
     & 2.6000000000D+02,
     & 9.5438088100D+03,
     & -6.8751750800D+01,
     & 4.0549219600D+00,
     & -3.0145573360D-04,
     & 3.7684977500D-06,
     & -2.2393589250D-09,
     & 3.0868591080D-13,
     & -3.2784572800D+03,
     & 1.4151946910D+00,
     & 1.4300402200D+06,
     & -5.2840286500D+03,
     & 1.0161821240D+01,
     & -9.7038499600D-04,
     & 2.1540034050D-07,
     & -2.1696957000D-11,
     & 9.3181630700D-16,
     & 2.9086962140D+04,
     & -4.3491603910D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpch4),i=1,mxprop)/1.6042500000D+01,
     & 3.3950000000D+01,
     & 3.7706000000D+01,
     & 2.2025704941D+00,
     & 3.5334737151D+01,
     & 1.5000000000D-06,
     & 2.0000000000D+00,
     & 1.0000000000D+00,
     & 3.0000000000D+00,
     & 2.0000000000D+00,
     & 5.0000000000D+00,
     & 1.0000000000D+00,
     & 5.3700000000D+02,
     & -1.7668509980D+05,
     & 2.7861810200D+03,
     & -1.2025778500D+01,
     & 3.9176192900D-02,
     & -3.6190544300D-05,
     & 2.0268530430D-08,
     & -4.9767054900D-12,
     & -2.3313143600D+04,
     & 8.9043227500D+01,
     & 3.7300427600D+06,
     & -1.3835014850D+04,
     & 2.0491070910D+01,
     & -1.9619747590D-03,
     & 4.7273130400D-07,
     & -3.7288146900D-11,
     & 1.6237372070D-15,
     & 7.5320669100D+04,
     & -1.2191248890D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00/
      data  (prop(i,jpc2h6),i=1,mxprop)/3.0069000000D+01,
     & 6.0430000000D+01,
     & 6.6067000000D+01,
     & 1.7061319331D+00,
     & 5.1301681095D+01,
     & 0.0000000000D+00,
     & 3.5000000000D+00,
     & 2.0000000000D+00,
     & 5.0000000000D+00,
     & 3.0000000000D+00,
     & 2.9000000000D+00,
     & 1.0000000000D+00,
     & 5.1500000000D+02,
     & -1.8620441610D+05,
     & 3.4061918600D+03,
     & -1.9517050920D+01,
     & 7.5658355900D-02,
     & -8.2041732200D-05,
     & 5.0611358000D-08,
     & -1.3192819920D-11,
     & -2.7029328900D+04,
     & 1.2981404960D+02,
     & 5.0257821300D+06,
     & -2.0330223970D+04,
     & 3.3225529300D+01,
     & -3.8367034100D-03,
     & 7.2384058600D-07,
     & -7.3191825000D-11,
     & 3.0654686990D-15,
     & 1.1159639500D+05,
     & -2.0394105840D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc3h8),i=1,mxprop)/4.4095600000D+01,
     & 8.6420000000D+01,
     & 9.3934000000D+01,
     & 1.6241540590D+00,
     & 7.1618047725D+01,
     & 0.0000000000D+00,
     & 5.0000000000D+00,
     & 3.0000000000D+00,
     & 7.0000000000D+00,
     & 4.0000000000D+00,
     & 2.0000000000D+00,
     & 1.0000000000D+00,
     & 4.6600000000D+02,
     & -2.4331443370D+05,
     & 4.6562708100D+03,
     & -2.9394660910D+01,
     & 1.1889527450D-01,
     & -1.3763082690D-04,
     & 8.8148239100D-08,
     & -2.3429879940D-11,
     & -3.5403352700D+04,
     & 1.8417492770D+02,
     & 6.4207316800D+06,
     & -2.6597911340D+04,
     & 4.5343568400D+01,
     & -5.0206639200D-03,
     & 9.4712169400D-07,
     & -9.5754052300D-11,
     & 4.0096728800D-15,
     & 1.4555824590D+05,
     & -2.8183747340D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpic4h10),i=1,mxprop)/5.8122200000D+01,
     & 1.1200000000D+02,
     & 1.2140000000D+02,
     & 1.6159346957D+00,
     & 9.3921679573D+01,
     & 0.0000000000D+00,
     & 6.5000000000D+00,
     & 4.0000000000D+00,
     & 9.0000000000D+00,
     & 5.0000000000D+00,
     & 1.8000000000D+00,
     & 1.0000000000D+00,
     & 4.0500000000D+02,
     & -3.8344693300D+05,
     & 7.0000396400D+03,
     & -4.4400269000D+01,
     & 1.7461834470D-01,
     & -2.0781953480D-04,
     & 1.3397924330D-07,
     & -3.5516816300D-11,
     & -5.0340188900D+04,
     & 2.6589664970D+02,
     & 7.5280189200D+06,
     & -3.2025170600D+04,
     & 5.7001610000D+01,
     & -6.0600130900D-03,
     & 1.1439758090D-06,
     & -1.1570618350D-10,
     & 4.8460429100D-15,
     & 1.7285008020D+05,
     & -3.5761768900D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpnc4h10),i=1,mxprop)/5.8122200000D+01,
     & 1.1240000000D+02,
     & 1.2179000000D+02,
     & 1.6538353804D+00,
     & 9.6124550747D+01,
     & 0.0000000000D+00,
     & 6.5000000000D+00,
     & 4.0000000000D+00,
     & 9.0000000000D+00,
     & 5.0000000000D+00,
     & 1.5000000000D+00,
     & 1.0000000000D+00,
     & 4.0500000000D+02,
     & -3.1758725400D+05,
     & 6.1763318200D+03,
     & -3.8915621200D+01,
     & 1.5846542840D-01,
     & -1.8600501590D-04,
     & 1.1996763490D-07,
     & -3.2016705500D-11,
     & -4.5403633900D+04,
     & 2.3794886650D+02,
     & 7.6823224500D+06,
     & -3.2560515100D+04,
     & 5.7367327500D+01,
     & -6.1979168100D-03,
     & 1.1801860480D-06,
     & -1.2218936980D-10,
     & 5.2506352500D-15,
     & 1.7745265600D+05,
     & -3.5879187600D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpic5h12),i=1,mxprop)/7.2148800000D+01,
     & 1.3810000000D+02,
     & 1.4936000000D+02,
     & 1.6004242713D+00,
     & 1.1546869067D+02,
     & 0.0000000000D+00,
     & 8.0000000000D+00,
     & 5.0000000000D+00,
     & 1.1000000000D+01,
     & 6.0000000000D+00,
     & 1.3000000000D+00,
     & 1.0000000000D+00,
     & 2.5800000000D+02,
     & -4.2319033900D+05,
     & 6.4971891000D+03,
     & -3.6811269700D+01,
     & 1.5324247290D-01,
     & -1.5487907140D-04,
     & 8.7498971200D-08,
     & -2.0705477100D-11,
     & -5.1554165900D+04,
     & 2.3095182180D+02,
     & 1.1568885940D+07,
     & -4.5562468700D+04,
     & 7.4954436300D+01,
     & -7.8454155800D-03,
     & 1.4443933140D-06,
     & -1.4643702130D-10,
     & 6.2302850000D-15,
     & 2.5449271350D+05,
     & -4.8019857800D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpnc5h12),i=1,mxprop)/7.2148800000D+01,
     & 1.3840000000D+02,
     & 1.4965000000D+02,
     & 1.6209949619D+00,
     & 1.1695284131D+02,
     & 0.0000000000D+00,
     & 8.0000000000D+00,
     & 5.0000000000D+00,
     & 1.1000000000D+01,
     & 6.0000000000D+00,
     & 1.4000000000D+00,
     & 1.0000000000D+00,
     & 2.5800000000D+02,
     & -2.7688946250D+05,
     & 5.8342834700D+03,
     & -3.6175414800D+01,
     & 1.5333397070D-01,
     & -1.5283958820D-04,
     & 8.1910920000D-08,
     & -1.7923279020D-11,
     & -4.6653752500D+04,
     & 2.2655440530D+02,
     & -2.5307792860D+06,
     & -8.9725932600D+03,
     & 4.5362232600D+01,
     & -2.6269899160D-03,
     & 3.1351364190D-06,
     & -5.3187289400D-10,
     & 2.8868968680D-14,
     & 1.4846165290D+04,
     & -2.5165503840D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpnc6h14),i=1,mxprop)/8.6175400000D+01,
     & 1.6439000000D+02,
     & 1.7755000000D+02,
     & 1.6117073622D+00,
     & 1.3888952662D+02,
     & 0.0000000000D+00,
     & 9.5000000000D+00,
     & 6.0000000000D+00,
     & 1.3000000000D+01,
     & 7.0000000000D+00,
     & 1.1000000000D+00,
     & 1.0000000000D+00,
     & 2.2300000000D+02,
     & -5.8159267000D+05,
     & 1.0790977240D+04,
     & -6.6339470300D+01,
     & 2.5237151550D-01,
     & -2.9043447050D-04,
     & 1.8022015140D-07,
     & -4.6172236800D-11,
     & -7.2715445700D+04,
     & 3.9382835400D+02,
     & -3.1066256840D+06,
     & -7.3460879200D+03,
     & 4.6941317600D+01,
     & 1.6939639770D-03,
     & 2.0689966670D-06,
     & -4.2121416800D-10,
     & 2.4523458450D-14,
     & 5.2375031200D+02,
     & -2.5499677180D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc7),i=1,mxprop)/1.0020190000D+02,
     & 1.9039000000D+02,
     & 2.0542000000D+02,
     & 1.6050314512D+00,
     & 1.6082720097D+02,
     & 0.0000000000D+00,
     & 1.1000000000D+01,
     & 7.0000000000D+00,
     & 1.5000000000D+01,
     & 8.0000000000D+00,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & 2.2300000000D+02,
     & -6.1274328900D+05,
     & 1.1840854370D+04,
     & -7.4871886000D+01,
     & 2.9184660520D-01,
     & -3.4167954900D-04,
     & 2.1592852690D-07,
     & -5.6558527300D-11,
     & -8.0134089400D+04,
     & 4.4072133200D+02,
     & 9.1356324700D+06,
     & -3.9233196900D+04,
     & 7.8897808500D+01,
     & -4.6542519300D-03,
     & 2.0717741420D-06,
     & -3.4425393000D-10,
     & 1.9768347750D-14,
     & 2.0507082950D+05,
     & -4.8511040200D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc8),i=1,mxprop)/1.1422850000D+02,
     & 2.0770000000D+02,
     & 2.2273000000D+02,
     & 1.6004300575D+00,
     & 1.8281472482D+02,
     & 0.0000000000D+00,
     & 1.2500000000D+01,
     & 8.0000000000D+00,
     & 1.7000000000D+01,
     & 9.0000000000D+00,
     & 8.0000000000D-01,
     & 1.0000000000D+00,
     & 2.2000000000D+02,
     & -6.9866471500D+05,
     & 1.3385010960D+04,
     & -8.4151659200D+01,
     & 3.2719366600D-01,
     & -3.7772095900D-04,
     & 2.3398369880D-07,
     & -6.0108926500D-11,
     & -9.0262232500D+04,
     & 4.9392221400D+02,
     & 6.3654069500D+06,
     & -3.1053646570D+04,
     & 6.9691623400D+01,
     & 1.0480596370D-02,
     & -4.1296219500D-06,
     & 5.5432263200D-10,
     & -2.6514364990D-14,
     & 1.5009687850D+05,
     & -4.1698956500D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc9),i=1,mxprop)/1.2724716000D+02,
     & 2.0942000000D+02,
     & 2.2069000000D+02,
     & 1.6074707517D+00,
     & 2.0454608794D+02,
     & 0.0000000000D+00,
     & 1.4000000000D+01,
     & 9.0000000000D+00,
     & 1.9000000000D+01,
     & 1.0000000000D+01,
     & 8.0000000000D-01,
     & 1.0000000000D+00,
     & 5.3000000000D+02,
     & -2.1948806120D+06,
     & 3.4685657800D+04,
     & -2.0092614190D+02,
     & 6.5805031100D-01,
     & -8.5259300100D-04,
     & 5.7946389600D-07,
     & -1.5976370990D-10,
     & -1.7231946080D+05,
     & 1.1491140170D+03,
     & 5.2773617400D+06,
     & -4.0257019600D+04,
     & 9.4572967200D+01,
     & -2.9403014470D-03,
     & 6.9781069900D-09,
     & 6.8052502400D-11,
     & -5.9070953300D-15,
     & 2.1653206140D+05,
     & -5.7544495000D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc10),i=1,mxprop)/1.4228170000D+02,
     & 2.1053000000D+02,
     & 2.1805000000D+02,
     & 8.9207008165D-01,
     & 1.2692524774D+02,
     & 0.0000000000D+00,
     & 1.5500000000D+01,
     & 1.0000000000D+01,
     & 2.1000000000D+01,
     & 1.1000000000D+01,
     & 9.0000000000D-01,
     & 1.0000000000D+00,
     & 5.2600000000D+02,
     & -2.6028453160D+05,
     & 6.2374095700D+03,
     & -5.2260950400D+01,
     & 2.3976927760D-01,
     & -2.9122448030D-04,
     & 1.8549444010D-07,
     & -4.8166192700D-11,
     & -1.1147008800D+04,
     & 2.9721395170D+02,
     & 5.9061721100D+06,
     & -3.1632292400D+04,
     & 7.0303420300D+01,
     & -6.0188655400D-03,
     & 1.1420521440D-06,
     & -1.1616056890D-10,
     & 4.8928440200D-15,
     & 1.9625670460D+05,
     & -4.3478489500D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc2h2),i=1,mxprop)/2.6037300000D+01,
     & 5.3159000000D+01,
     & 5.5038000000D+01,
     & 1.6604691511D+00,
     & 4.3234133427D+01,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 2.0000000000D+00,
     & 3.0000000000D+00,
     & 1.0000000000D+00,
     & 1.5000000000D+00,
     & 1.0000000000D+00,
     & 3.0500000000D+02,
     & 1.5981120890D+05,
     & -2.2166441180D+03,
     & 1.2657078130D+01,
     & -7.9796510800D-03,
     & 8.0549927500D-06,
     & -2.4333076730D-09,
     & -7.5292331800D-14,
     & 3.7126190600D+04,
     & -5.2443389000D+01,
     & 1.7138474100D+06,
     & -5.9291066600D+03,
     & 1.2361279430D+01,
     & 1.3141869930D-04,
     & -1.3627644310D-07,
     & 2.7126557860D-11,
     & -1.3020662040D-15,
     & 6.2665789700D+04,
     & -5.8189605900D+01,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc2h4),i=1,mxprop)/2.8053200000D+01,
     & 5.5960000000D+01,
     & 5.9721000000D+01,
     & 1.4950764390D+00,
     & 4.1941678358D+01,
     & 0.0000000000D+00,
     & 3.0000000000D+00,
     & 2.0000000000D+00,
     & 4.0000000000D+00,
     & 2.0000000000D+00,
     & 2.9000000000D+00,
     & 1.0000000000D+00,
     & 4.9000000000D+02,
     & -1.1636058360D+05,
     & 2.5548515100D+03,
     & -1.6097464280D+01,
     & 6.6257793200D-02,
     & -7.8850818600D-05,
     & 5.1252248200D-08,
     & -1.3703400310D-11,
     & -6.1761910700D+03,
     & 1.0933383430D+02,
     & 3.4087636700D+06,
     & -1.3748479030D+04,
     & 2.3658980740D+01,
     & -2.4238044190D-03,
     & 4.4313956600D-07,
     & -4.3526833900D-11,
     & 1.7754106330D-15,
     & 8.8204293800D+04,
     & -1.3712781080D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc3h6),i=1,mxprop)/4.2079700000D+01,
     & 8.1460000000D+01,
     & 8.7097000000D+01,
     & 1.4941204336D+00,
     & 6.2872139612D+01,
     & 0.0000000000D+00,
     & 4.5000000000D+00,
     & 3.0000000000D+00,
     & 6.0000000000D+00,
     & 3.0000000000D+00,
     & 2.1000000000D+00,
     & 1.0000000000D+00,
     & 4.6000000000D+02,
     & -1.9124621740D+05,
     & 3.5420742400D+03,
     & -2.1148786260D+01,
     & 8.9014847900D-02,
     & -1.0014291540D-04,
     & 6.2679593900D-08,
     & -1.6378707810D-11,
     & -1.5299618240D+04,
     & 1.4076413820D+02,
     & 5.0176203400D+06,
     & -2.0860840350D+04,
     & 3.6441563400D+01,
     & -3.8811911700D-03,
     & 7.2786771900D-07,
     & -7.3212045000D-11,
     & 3.0521763690D-15,
     & 1.2612453550D+05,
     & -2.1957157570D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc4h8),i=1,mxprop)/5.6106300000D+01,
     & 1.0746000000D+02,
     & 1.1498000000D+02,
     & 1.4850191398D+00,
     & 8.3318929363D+01,
     & 0.0000000000D+00,
     & 6.0000000000D+00,
     & 4.0000000000D+00,
     & 8.0000000000D+00,
     & 4.0000000000D+00,
     & 1.6000000000D+00,
     & 1.0000000000D+00,
     & 3.8500000000D+02,
     & -2.7214920140D+05,
     & 5.1000792500D+03,
     & -3.1837862500D+01,
     & 1.3177544420D-01,
     & -1.5273593390D-04,
     & 9.7147611100D-08,
     & -2.5602044700D-11,
     & -2.5230963860D+04,
     & 2.0069321080D+02,
     & 6.2579486100D+06,
     & -2.6603763050D+04,
     & 4.7649200500D+01,
     & -4.3832671100D-03,
     & 7.1288384400D-07,
     & -5.9910208400D-11,
     & 2.0517535040D-15,
     & 1.5692526570D+05,
     & -2.9138697610D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc5h10),i=1,mxprop)/7.0132900000D+01,
     & 1.3346000000D+02,
     & 1.4285000000D+02,
     & 1.5021753328D+00,
     & 1.0535191240D+02,
     & 0.0000000000D+00,
     & 7.5000000000D+00,
     & 5.0000000000D+00,
     & 1.0000000000D+01,
     & 5.0000000000D+00,
     & 1.3000000000D+00,
     & 1.0000000000D+00,
     & 2.4500000000D+02,
     & -5.3405481300D+05,
     & 9.2989173800D+03,
     & -5.6677924500D+01,
     & 2.1231002660D-01,
     & -2.5712982900D-04,
     & 1.6668343040D-07,
     & -4.4340804700D-11,
     & -4.7906821800D+04,
     & 3.3960364000D+02,
     & 3.7440149700D+06,
     & -2.1044853210D+04,
     & 4.7361269900D+01,
     & -4.2442012000D-04,
     & -3.8989750500D-08,
     & 1.3670742430D-11,
     & -9.3131942300D-16,
     & 1.1540913730D+05,
     & -2.7861774490D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpc6h12),i=1,mxprop)/8.4159500000D+01,
     & 1.5604000000D+02,
     & 1.6731000000D+02,
     & 1.2027467730D+00,
     & 1.0122256704D+02,
     & 0.0000000000D+00,
     & 9.0000000000D+00,
     & 6.0000000000D+00,
     & 1.2000000000D+01,
     & 6.0000000000D+00,
     & 1.2000000000D+00,
     & 1.0000000000D+00,
     & 2.4500000000D+02,
     & -5.6799870400D+05,
     & 1.0342387040D+04,
     & -6.8000412500D+01,
     & 2.3877976580D-01,
     & -2.5118900490D-04,
     & 1.4252931840D-07,
     & -3.4078331900D-11,
     & -6.4046351600D+04,
     & 3.9348082100D+02,
     & 5.2251494700D+06,
     & -3.3641945800D+04,
     & 7.1746074700D+01,
     & -6.6989791200D-03,
     & 1.3184432540D-06,
     & -1.3907947890D-10,
     & 6.0601022400D-15,
     & 1.7325376090D+05,
     & -4.5468141700D+02,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpco),i=1,mxprop)/2.8010100000D+01,
     & 1.1965000000D+01,
     & 1.1965000000D+01,
     & 1.0401293823D+00,
     & 2.9134128010D+01,
     & 2.0000000000D-07,
     & 5.0000000000D-01,
     & 1.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.2500000000D+01,
     & 1.0000000000D+00,
     & 6.0900000000D+02,
     & 1.4890453260D+04,
     & -2.9222859390D+02,
     & 5.7245271700D+00,
     & -8.1762350300D-03,
     & 1.4569034690D-05,
     & -1.0877463020D-08,
     & 3.0279418270D-12,
     & -1.3031318780D+04,
     & -7.8592413500D+00,
     & 4.6191972500D+05,
     & -1.9447048630D+03,
     & 5.9167141800D+00,
     & -5.6642828300D-04,
     & 1.3988145400D-07,
     & -1.7876803610D-11,
     & 9.6209355700D-16,
     & -2.4662610840D+03,
     & -1.3874131080D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & 1.0000000000D+00/
      data  (prop(i,jpnh3),i=1,mxprop)/1.7030500000D+01,
     & 1.3401000000D+01,
     & 1.6220000000D+01,
     & 2.0773727532D+00,
     & 3.5378696674D+01,
     & 0.0000000000D+00,
     & 7.5000000000D-01,
     & 0.0000000000D+00,
     & 2.0000000000D+00,
     & 1.5000000000D+00,
     & 1.5500000000D+01,
     & 1.0000000000D+00,
     & 6.3000000000D+02,
     & -7.6812261500D+04,
     & 1.2709515780D+03,
     & -3.8932291300D+00,
     & 2.1459884180D-02,
     & -2.1837667030D-05,
     & 1.3173857060D-08,
     & -3.3323220600D-12,
     & -1.2648864130D+04,
     & 4.3660145880D+01,
     & 2.4523895350D+06,
     & -8.0408942400D+03,
     & 1.2713462010D+01,
     & -3.9801865800D-04,
     & 3.5525027500D-08,
     & 2.5309235700D-12,
     & -3.3227005300D-16,
     & 4.3861919600D+04,
     & -6.4623306020D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpar),i=1,mxprop)/3.9948000000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 5.2033034308D-01,
     & 2.0786156545D+01,
     & 9.3320000000D-03,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 4.3796749100D+00,
     & 2.0105384750D+01,
     & -5.9926610700D-02,
     & 2.5000694010D+00,
     & -3.9921411600D-08,
     & 1.2052721400D-11,
     & -1.8190155760D-15,
     & 1.0785766360D-19,
     & -7.4499396100D+02,
     & 4.3791801100D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpo2),i=1,mxprop)/3.1998800000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 9.1665963096D-01,
     & 2.9332008199D+01,
     & 2.0939000000D-01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -3.4255634200D+04,
     & 4.8470009700D+02,
     & 1.1190109610D+00,
     & 4.2938892400D-03,
     & -6.8363005200D-07,
     & -2.0233727000D-09,
     & 1.0390400180D-12,
     & -3.3914548700D+03,
     & 1.8496994700D+01,
     & -1.0379390220D+06,
     & 2.3448302820D+03,
     & 1.8197320360D+00,
     & 1.2678475820D-03,
     & -2.1880679880D-07,
     & 2.0537195720D-11,
     & -8.1934670500D-16,
     & -1.6890109290D+04,
     & 1.7387165060D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & -9.9990000000D+03/
      data  (prop(i,jpne),i=1,mxprop)/2.0179700000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0300528028D+00,
     & 2.0786156545D+01,
     & 1.8200000000D-05,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 3.3553227200D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 3.3553227200D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpkr),i=1,mxprop)/8.3800000000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.4804482751D-01,
     & 2.0786156545D+01,
     & 1.1000000000D-06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 5.4909565100D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4034889400D+02,
     & 5.4843971500D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpn2o),i=1,mxprop)/4.4012800000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 8.6719224654D-01,
     & 3.8167558908D+01,
     & 3.0000000000D-07,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 4.2882259700D+04,
     & -6.4401184400D+02,
     & 6.0343514300D+00,
     & 2.2653944360D-04,
     & 3.4727828500D-06,
     & -3.6277486400D-09,
     & 1.1379695520D-12,
     & 1.1794055060D+04,
     & -1.0031285700D+01,
     & 3.4384480400D+05,
     & -2.4045575580D+03,
     & 9.1256362200D+00,
     & -5.4016679300D-04,
     & 1.3151240310D-07,
     & -1.4142151000D-11,
     & 6.3810668700D-16,
     & 2.1986326380D+04,
     & -3.1478050160D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpxe),i=1,mxprop)/1.3129300000D+02,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.5831884827D-01,
     & 2.0786156545D+01,
     & 1.0000000000D-07,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 2.5000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -7.4537500000D+02,
     & 6.1644542050D+00,
     & 4.0252266800D+03,
     & -1.2095075210D+01,
     & 2.5141533470D+00,
     & -8.2481020800D-06,
     & 2.5302326180D-09,
     & -3.8923332300D-13,
     & 2.3604391380D-17,
     & -6.6858007300D+02,
     & 6.0637107150D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & 1.0000000000D+00/
      data  (prop(i,jpso2),i=1,mxprop)/6.4063800000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 6.1626833741D-01,
     & 3.9480491514D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -5.3108421400D+04,
     & 9.0903116700D+02,
     & -2.3568912440D+00,
     & 2.2044498850D-02,
     & -2.5107814710D-05,
     & 1.4463004840D-08,
     & -3.3690709400D-12,
     & -4.1137520800D+04,
     & 4.0455125190D+01,
     & -1.1276401160D+05,
     & -8.2522613800D+02,
     & 7.6161786300D+00,
     & -1.9993276100D-04,
     & 5.6556314300D-08,
     & -5.4543166100D-12,
     & 2.9182941020D-16,
     & -3.3513086900D+04,
     & -1.6557760850D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jps),i=1,mxprop)/3.2066000000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 7.3900633135D-01,
     & 2.3696977021D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0000000000D+06,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & -3.1748418200D+02,
     & -1.9247049230D+02,
     & 4.6868259300D+00,
     & -5.8413656000D-03,
     & 7.5385335200D-06,
     & -4.8635860400D-09,
     & 1.2569769920D-12,
     & 3.3235921800D+04,
     & -5.7185239690D+00,
     & -4.8542447900D+05,
     & 1.4388304080D+03,
     & 1.2585041160D+00,
     & 3.7979904300D-04,
     & 1.6306858640D-09,
     & -9.5470958500D-12,
     & 8.0414666460D-16,
     & 2.3349952700D+04,
     & 1.5595548550D+01,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpcos),i=1,mxprop)/6.0075100000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 6.8293498107D-01,
     & 4.1027387281D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 8.5478764300D+04,
     & -1.3194648210D+03,
     & 9.7352572400D+00,
     & -6.8708309600D-03,
     & 1.0823314160D-05,
     & -7.7055973400D-09,
     & 2.0785703440D-12,
     & -1.1916576850D+04,
     & -2.9919885930D+01,
     & 1.9590985670D+05,
     & -1.7561676880D+03,
     & 8.7104303400D+00,
     & -4.1394249600D-04,
     & 1.0152436480D-07,
     & -1.1596096630D-11,
     & 5.6910538600D-16,
     & -8.9270966900D+03,
     & -2.6363280160D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpcs2),i=1,mxprop)/7.6140700000D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 5.9097978084D-01,
     & 4.4997614199D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.6135604820D+04,
     & -4.6494814700D+02,
     & 6.2979387900D+00,
     & 1.8888967060D-03,
     & 3.0319277470D-07,
     & -1.7376453730D-09,
     & 7.7939893900D-13,
     & 1.4777611190D+04,
     & -9.3033821300D+00,
     & -1.3904197240D+06,
     & 3.3549755000D+03,
     & 3.0192477230D+00,
     & 2.8764375430D-03,
     & -9.0768127200D-07,
     & 1.3740910420D-10,
     & -6.9995755700D-15,
     & -1.0138980460D+04,
     & 1.5651137030D+01,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpcl2),i=1,mxprop)/7.0906000000D+01,
     & 2.3967000000D+01,
     & 2.3967000000D+01,
     & 4.7626224000D-01,
     & 3.3769850389D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 3.4628151700D+04,
     & -5.5471265200D+02,
     & 6.2075893700D+00,
     & -2.9896320780D-03,
     & 3.1730272900D-06,
     & -1.7936295620D-09,
     & 4.2600435900D-13,
     & 1.5340693310D+03,
     & -9.4383311070D+00,
     & 6.0925694200D+06,
     & -1.9496276620D+04,
     & 2.8545357950D+01,
     & -1.4499687640D-02,
     & 4.4638907700D-06,
     & -6.3585258600D-10,
     & 3.3273602900D-14,
     & 1.2121177240D+05,
     & -1.6907788240D+02,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & -9.9990000000D+03,
     & -9.9990000000D+03/
      data  (prop(i,jpair),i=1,mxprop)/2.8965515609D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0042834317D+00,
     & 2.9089587418D+01,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 0.0000000000D+00,
     & 1.0106464291D+04,
     & -1.9692061904D+02,
     & 5.0095226420D+00,
     & -5.7612223529D-03,
     & 1.0668750278D-05,
     & -7.9402700959D-09,
     & 2.1851882911D-12,
     & -1.8019159530D+02,
     & -4.4906610669D+00,
     & 2.4163504688D+05,
     & -1.2582739997D+03,
     & 5.1451613534D+00,
     & -2.1398314782D-04,
     & 7.0674550883D-08,
     & -1.0717110578D-11,
     & 6.5792025782D-16,
     & 6.4608578347D+03,
     & -8.7184622599D+00,
     & 1.0000000000D+00,
     & -9.9990000000D+03,
     & 1.0000000000D+00,
     & -9.9990000000D+03/


      
      end module mod_globals 
      

c-----------------------------------------------------------------------
      module mod_virtualsources
c-----------------------------------------------------------------------
      use mod_stddefs 

      integer :: indx(mxmet)  ! index
      real(8) :: vrise(mxmet) ! plume rise, m
      real(8) :: vdex(mxmet)  ! source diameter, m
      real(8) :: vhsx(mxmet)  ! source height, m
      real(8) :: vx(mxmet)  ! source x-location, m
      real(8) :: vy(mxmet)  ! source y-location, m
      real(8) :: ve(mxmet)   ! source emission rate, g/s
      real(8) :: vusx(mxmet)   ! source exit velociy, m/s
      real(8) :: vtsx(mxmet)  ! source temperature,K
      integer :: ivs(mxmet)   ! source #
      real(8) :: vfm(mxmet)  ! source  fm
      real(8) :: vfb(mxmet)  ! source  fb
      real(8) :: vta(mxmet)   ! save used ambient temp
      real(8) :: vua(mxmet)      ! save used ua
      integer :: vdate(mxmet)
      integer :: vsec(mxmet)
      
      real(8) :: vsTs(mxvsrc)  ! average Ts
      real(8) :: vsUs(mxvsrc)  ! average Us
      real(8) :: vsHs(mxvsrc)  ! average hs
      real(8) :: vsDs(mxvsrc)  ! average ds
      real(8) :: vsX(mxvsrc)  ! average x
      real(8) :: vsY(mxvsrc)  ! average y
      integer :: ivsNs(mxvsrc) ! number of in group
      integer  :: nmet
      
      end module mod_virtualsources


      
      
c-----------------------------------------------------------------------
      module mod_file1
c-----------------------------------------------------------------------
      use mod_stddefs 
      
      end module mod_file1

      

c-----------------------------------------------------------------------
      module mod_readinp
c-----------------------------------------------------------------------
      use mod_stddefs
      
c	Source description type 
c
c  MDSPMOD=1 : emissions file output is for CALPUFF
c          2 : emissions file output is for AERMOD
c
c      
c  MINPUT =1 : source  conditions are input and 
c	               blowdown is calculated (see MBLOWDOWN)
c	           2 : blowdown curve sequence is input
c			   3 : static source is input (see GROUP SOURCE)
c		default : 1
c
c                    only used if MINPUT =1
c MBLOWDOWN=   0: Steady 
c	   		     1 : blowdown exponential model 
c	   				 parameters are calculated from pipeline/vessel 
c	   				 initial conditions
c              2: user input QMAX and QTOTAL
c              3: user input QAVE (as QMAX), PRatio and TDEP
c              4: user input QMAX, PRatio and TDEP
c              5: user input QAVE (as QMAX), TDEP and Pratio=50%
c		default : 0  
c
c		only used if MINPUT =1
c MDIST=1 : blowdown sequence is created based upon
c				 puffs of equal volume
c	         2 : blowdown sequence is created based upon
c				 puffs of equal mass
c		default : 2  
c
c MSTRIP=0 : source is described by input (100% conversion to SO2)
c	         =1 : source is a stripped flow source (H2S)
c			 =2 : source is actual SO2 (accounting for efficiency)
c
c MMET=1:  meteorology is read from METPATH, METSERIES style
c     =2:  meteorology is read from METPATH, aermet style
c     =3:  force met to be WINDSP, WINDDR, TAMB 
c
c MFUELGAS: Is fuel gas added?
c		0 = no fuel gas is added
c		1 = fuel gas is added
c
c MLIFTGAS: Is lift gas added?
c		0 = no lift gas is added
c		1 = lift gas is added
c
c 
c MFGR =1 : fuel gas is proportional to qmax rate 
c            =2 : fuel gas is constant amount based upon initial qmax  
c		default : 2
c
c
c MFASSIST
c      Is flare assist used ?
c		0 = no flare assist
c		1 = yes flare assist
c MFASSISTSTEAM
c		0 = no flare assist
c		1 = yes flare assist
c MFASSISTAIR
c		0 = no flare assist
c		1 = yes flare assist
c MFASSISTHPFUEL      
c		0 = no flare assist
c		1 = yes flare assist
c
c MTGAS
c        0= tgas=tginit
c        1= tgas=tamb
c        2= tgas=Tei calculated from vessel blowdown

      integer :: MDSPMOD         ! output is for 1:CALPUFF; 2:AERMOD
      integer :: MINPUT         ! 
      integer :: MBLOWDOWN      ! 
      integer :: MDIST          !
      integer :: MSTRIP         !
      integer :: MMET           !
	integer :: MFUELGAS		! 0: no fuel gas is added; 1:user fuel gas is added
      integer :: MFGR           !
	integer :: MLIFTGAS		! 0: no lift gas ; 1:liftgas is added
	integer :: MFASSIST		! iflareassist:0 no assist; 1=steam; 2=air; 3=both
	integer :: MFASSISTSTEAM		! iflareassist:0 no assist; 1=steam; 2=air; 3=both
	integer :: MFASSISTAIR		! iflareassist:0 no assist; 1=steam; 2=air; 3=both
	integer :: MFASSISTHPFUEL   ! iflareassist:0 no assist; 1=steam; 2=air; 3=both
      integer :: MTGAS        ! 0: tgas=tginit
                              ! 1: tgas=tamb
                              ! 2: tgas=Tei calculated from vessel blowdown
	integer :: MSCREENING  	! 0: no screening 1: screening, met is discretized for AERMOD
	integer :: MDEBUG			! 0: no debug   1: debug
	integer :: MNONSEQ    	! 0: dates are sequential   1: screening, AERMODE dates may be non sequential
	integer :: MSCREEN     	! 0: no screening 1: screening, AERMOD screening mode
      integer :: MFLAMEOUT    ! 0: combusion  1: flameout (model H2S)  2: flameout (model raw gas, flambility)
      integer :: MGROUNDFLARE ! 0: no 1: source is a multipoint ground flare
      integer :: MDLIFT       !
      integer :: MDFUEL       !
      
      
      integer :: nheads       ! number of flare heads when MGROUNDFLARE==1

      integer :: iyys, imms, idds, ihhs, isss, ijjs, idatestart
      integer :: iyye, imme, idde, ihhe, isse, ijje, idateend
      integer :: itiminc
      integer :: nspec
      real(8) :: mmlst(mxchem)  ! mole weight list 
      real(8) :: qlst(mxchem)   ! emissions list
      real(8) :: xcoord			! location of source km-UTM
      real(8) :: ycoord			! location of source km-UTM
      real(8) :: zcoord			! elevation of source m
      real(8)  :: hs               ! actual height,m 
      real(8)  :: df               ! actual flare top diameter, m
      real(8)  :: ds               ! unobstructed flow diameter, m
      real(8)  :: dlift    ! lift gas line inside diameter (m)
      real(8)  :: dfuel    ! fuel line inside diameter (m)
      
      real(8)  :: pshs             ! pseudo-height,m
      real(8)  :: psds             ! pseudo-diameter, m
      real(8)  :: psus             ! pseudo-velocity, m/s
      real(8)  :: psts             ! pseudo-temperature, K
      real(8)  :: rh2s             ! mole fraction of H2S
      real(8)  :: tginit		!  Initial flared (raw and fuel) Gas Temperature	K 
      real(8)  :: qmax             ! peak rate for blowdown curve, e3m3/d
      real(8)  :: qtotal           ! total volume in pipeline/vessel, e3m3
      real(8)  :: duration         ! duration of flare event, h
      real(8)  :: pratio           ! depressurization ratio, 50% in 15min
      real(8)  :: tdep             ! time to depressurization ratio, min
      
      real(8)  :: frel             ! fraction left in pipeline/vessel
	real(8) :: pressinit		!  Expected Maximum Initial Pressure	kPa (gauge)
      real(8) :: tempinit		!  Expected Minimum Initial Gas Temperature	K 
	real(8) :: pressfinal		!  Expected Minimum Final Pressure	kPa
	real(8) :: vesseldia		!  Pipeline/Vessel Inside Diameter	m
	real(8) :: vessellen		!  Pipeline/Vessel Length	m
	real(8) :: orificedia		!  Minimum Orifice Diameter	mm
	real(8) :: dischargecoef	!  Discharge Coefficient	
      real(8) :: odourppb		! odour threshold [ppb] for calculation of odour units emissions

      real(8)  :: rWINDSP     ! user wind speed static, m
      real(8)  :: rWINDDR     ! user wind dir static, m/s 
      real(8)  :: rTAMB       ! user temperature ambient static K
      real(8)  :: rqemit      ! emission rate, kg/s
      real(8)  :: zref       ! user zref, m
      integer  :: irPGCLASS
      integer  :: ipuffdur    ! number of seconds duration
      integer  :: npuffs
      real(8)  :: fgr
      real(8)  :: gcraw(mxcomp)
      real(8)  :: gcfuel(mxcomp)
      real(8)  :: gclift(mxcomp)
      real(8)  :: pseq(mxpuff,4)

      real(8) :: qliftgas	! lift gas flow rate e3m3/d
      real(8) :: stm_press		! steam assist pressure of steam or -1 if unknown
      real(8) :: stm_temp		! steam assist temperature of steam or -1 if unknown
      real(8) :: stm_flow		! steam assist mass flow rate kg/s or flow rate e3m3/d
      real(8) :: rairair      ! air assist : air to air entrainment factor, 0.35
      real(8) :: rairsteam    ! steam assist : air to steam entrainment factor, 0.35
      real(8) :: rairfuel     ! fuel assist : air to fuel entrainment factor, 0.35
	real(8) :: air_flow		! air assist flow rate e3m3/d
	real(8) :: hpfuel_flow		! air assist flow rate e3m3/d
	integer :: stm_imassvol	! steam assist is 1=mass flow; 2=volume flow
      integer :: NSEP		! separation 24,48,72,...
	integer :: nsources	! number of aermod sources
	integer :: nseq		! length of user defined blowdown sequence
	integer :: iutmzn		! utm zone

      character*128 :: stitle
      character*32 :: sname
      CHARACTER*1 :: UTMHEM
      CHARACTER*6 :: PMAP
      CHARACTER*6 :: DATUM
      character*8 :: ABTZ
      character*8 :: chemlst(mxchem)
      character*mxpath :: METDAT
      character*mxpath :: METDAT2
      character*mxpath :: RECINC
      character*mxpath :: OUTNAME
      character*mxpath :: OUTPATH
      character*mxpath :: LSTDAT
      character*mxpath :: LOGDAT

      logical :: lcfiles	! T: convert all files to lower case
							! F: do not

      integer :: MUNITS				! units for coordinates in FLarb file

      real(8) :: gcmix(mxcomp)   ! gas composition mixed
      real(8) :: gcraw_x(mxcomp) ! gas composition raw
      real(8) :: q(mxchem)       ! flare rate vector
      real(8) :: DELH(5)      ! plume rise (m)
      real(8) :: XH(5)        ! distance to final rise (m)
      real(8) :: tgas2        ! Temp of flared gas calc'd by blowdown
      real(8) :: ws,wd,wsreal,tambreal,qmax0

      integer :: ipuffdurx    ! puff duration (sec)
      integer :: idatecur     ! current date
      integer :: iblowstart   ! start of a blowdown cycle
      integer :: ipg,ipgreal,icase
      
      end module mod_readinp
      
c-----------------------------------------------------------------------
      module mod_isccalcs
c-----------------------------------------------------------------------

      real(8) :: FBISC      ! FB calculated by ISC
      real(8) :: FMISC      ! FM calculated by ISC
      real(8) :: RTOFS       ! root of S
      real(8) :: XFM         ! distance to final momentum rise (m)
      real(8) :: XFB         ! distance to final buooyancy rise (m)
      logical :: UNSTAB
      logical :: NEUTRL
      logical :: STABLE

      end module mod_isccalcs      
      
      
c-----------------------------------------------------------------------
      module mod_combustion
c-----------------------------------------------------------------------

      real(8) :: e_so2_m  ! emission rate of SO2 (g/s)
      real(8) :: e_so2_ma ! actual emission rate of SO2 (g/s)
      real(8) :: e_h2s_m  ! emission rate of H2S (g/s)
      real(8) :: e_so2_m1 ! emission rate of SO2 (t/d)
      real(8) :: e_co2_m1 ! emission rate of CO2 (t/d)
      real(8) :: FM   ! momentum flux
      real(8) :: FB   ! buoyancy flux
      real(8) :: extemp ! calculated exhaust temperature of flame
      real(8) :: exrate ! calculated exhaust rate m3/s
      real(8) :: qemit  ! calculated emission rate (g/s)
      real(8) :: hs_x   ! pseudo height
      real(8) :: ue_x   ! pseudo velocity
      real(8) :: de_x   ! pseudo diameter
      real(8) :: den_mix ! exhaust density
      real(8) :: den_air ! air density
      real(8) :: XL  !
      real(8) :: ZL  !
      real(8) :: FA  !
      real(8) :: hf  ! flame height above stack tip
      real(8) :: uu  ! horizontal velocity
      real(8) :: uv  ! vertical velocity
      real(8) :: plumeangle ! plume angle in vertical
      real(8) :: conv_so2  ! in-plume so2 concentration ppm
      real(8) :: resid_h2s ! in-plume H2S concentration ppm
      real(8) :: eta  ! conversion in-efficiency
	integer :: isflammable	! 0: nonlammable 1:flammable

      end module mod_combustion      
      
      
c-----------------------------------------------------------------------
      module mod_met
c-----------------------------------------------------------------------

      real(8) :: zws        ! height of wind speed
      real(8) :: zwd        ! height of wind direction

      integer :: indws     ! record number for wind speed
      integer :: indwd     ! record number for wind direction
      integer :: indt      ! record number for temperature
      integer :: indpg     ! record number for PG stability
      integer :: indmo     ! record number for monin-obikoff

      integer :: iwsunits  ! units type for wind speed
      integer :: itunits   ! units type for temperature

      integer :: jdatebm   ! met file start date
      integer :: isecbm    ! met file start seconds
      integer :: jdateem   ! met file end date
      integer :: isecem    ! met file end seconds
      integer :: jdatebmi  ! current met record start date 
      integer :: isecbmi   ! current met record start seconds 
      integer :: jdateemi  ! current met record end  date 
      integer :: isecemi   ! current met record end seconds

      end module mod_met
      
      