c  token.for
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c     See also VBA code for added tokenizers to handle
c       -- replacing a list with same token name
c              eg,  METDAT=<file1>
c                   METDAT=<file2>       
c       -- search for token name and replace its value inplace
c       -- more complex matrix      
c       
c  These routines use basic fortran intrinsic functions for 
c  tokenizing and extracting data using the CALPUFF style
c  input files
c
c  NOTE: !END! are always ignored, since they are redundant      
c    assuming all input parameters are unique
c
c  -input files are ASCII text
c  -parameter are in the follow format
c	 comment ! A = 1 ! comment
c
c	! A = TEXT !                  irdparam_a( )
c	! A = TEXT1,TEXT2,TEXT3 !     irdparam_av( )
c	! nread = 3    !              irdparam_av2( )
c    	! TEXT1,TEXT2 !     
c    	! TEXT1,TEXT2 !     
c    	! TEXT1,TEXT2 !     
c
c	! A = 1!                  irdparam_i( )
c	! A = 1,2,3 !             irdparam_iv( )
c	! nread = 3    !          irdparam_iv2( )
c    	! 1,2 !     
c    	! 1,2 !     
c    	! 1,2 !     
c
c	! A = 1.234!                irdparam_r( )
c	! A = 1.231,1.23,1.23 !     irdparam_rv( )
c	! nread = 3    !            irdparam_rv2( )
c    	! 1.23,1.23 !     
c    	! 1.23,1.23 !     
c    	! 1.23,1.23 !     
c
c Higher Level User Routines      
c     irdparam_a( )       : returns the text token
c     irdparam_av( )      : returns a vector of text tokens
c     irdparam_av2( )     : returns a matrix of text tokens
c     irdparam_i( )       : returns the integer token
c     irdparam_iv( )      : returns a vector of integer tokens
c     irdparam_iv2( )     : returns a matrix of integer tokens
c     irdparam_r( )       : returns the real token
c     irdparam_rv( )      : returns a vector of real tokens
c     irdparam_rv2( )     : returns a matrix of real tokens
c
c Low level       
c     LEQ( )              : determines if two strings are equivalent
c     len_real( )         : returns the actual length of the string
c     zstrcopy( )         : copies string a to b
c     zstrtrim( )         : removes blanks from string
c     itokenalist( )      : returns a list of text
c     itokenptr( )        : finds the start location
c     itoken2i( )         : converts a token to integer
c     itoken2r( )         : converts a token to real
c     itokenize( )        : splits the string into parts, contols, name, token value    



c      Interface
c          sum zstrtrim(str)
c            character *(*)::str
c            character *(*)::zstrtrim
c         end function
c      end interface

c ---      PROGRAM Example
c ---      CHARACTER(60) :: str = "aaaa !tim=1,2,3,4,5,6! bbbb"  
c ---      CHARACTER(60) :: str2 
c ---      CHARACTER(60) :: str3 
c ---      CHARACTER *255 :: sparam
c ---      CHARACTER *255 :: svalue
c ---      CHARACTER(5)   :: word(5)  
c ---      INTEGER :: pos1 = 1, pos2, n = 0, i, pos
c ---      integer :: ilist(10)
c ---      real     :: rlist(10)
c ---
c ---      write(*,*) str
c ---      pos =1
c ---      if( itokenize( str, sparam, svalue) .gt. 0) then
c ---c      if( itokenptr(str, pos, "!") .gt. 0) then 
c ---c         pos =1
c ---c         pos = itoken(str, pos, "!", str2)
c ---cc         write(*,*) str2, pos
c ---c         pos = itoken(str, pos, "!", str2)
c ---c         write(*,*) trim(str2)
c ---c         pos =1
c ---c         if( itokenptr(str2, pos, "=") .gt. 0) then 
c ---c            pos =1
c ---c            pos = itoken(str2, pos, "=", str3)
c ---c            write(*,*) "Parameter: [",trim(str3),"]"
c ---c            pos = itoken(str2, pos, "=", str3)
c ---c            write(*,*) "Value(s):", trim(str3)
c ---c            pos =1
c ---cc            n = itokenilist(str3, pos, ",", ilist)
c ---c            n = itokenrlist(str3, pos, ",", rlist)
c ---c            write(*,*) "LIST"
c ---c            DO i = 1, n
c ---cc               WRITE(*,"(2A)", ADVANCE="NO") TRIM(word(i)), "." 
c ---c               WRITE(*,*) rlist(i)
c ---c            END DO 
c ---c         end if
c ---            n = itokenrlist(svalue, pos, ",", rlist)
c ---            write(*,*) "LIST"
c ---            DO i = 1, n
c ---               WRITE(*,*) rlist(i)
c ---            END DO 
c ---      end if 
c ---
c ---c      DO
c ---c         pos2 = INDEX(str(pos1:), ",")
c ---c         IF (pos2 .eq. 0) THEN
c ---c            n = n + 1
c ---c            word(n) = str(pos1:)
c ---c            EXIT
c ---c         END IF
c ---c         n = n + 1
c ---c         word(n) = str(pos1:pos1+pos2-2)
c ---c         pos1 = pos2+pos1 
c ---c      END DO  
c ---c      DO i = 1, n
c ---c         WRITE(*,"(2A)", ADVANCE="NO") TRIM(word(i)), "." 
c ---c      END DO 
c ---c	  END IF
c ---      stop
c ---      END PROGRAM Example

c----------------------------------------------------------------------
c     itoken2i
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     reads an integer from the str based upon the real length of str         
c
c --- UPDATES:
c --- INPUT:
c --- OUTPUT:
c 		i: returns integer
c		ier: function call returns 1 if integer was read or zero if nothing
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itoken2i(str, i)
     &   result(ierr)
      character *(*):: str
      integer  :: i, n
      integer  :: ierr
      ierr = 0
      n = len_real(str)
      read(str(1:n),*,err=9999) i
      ierr = 1
9999  return
      end

c----------------------------------------------------------------------
c     itoken2r
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     reads a real from the str based upon the real length of str          
c
c --- UPDATES:
c --- INPUT:
c --- OUTPUT:
c 		r: returns integer
c		ier: function call returns 1 if real was read or zero if nothing
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itoken2r(str, r)
     &   result(ierr)
      character *(*):: str
      real(8)     :: r
      integer  :: ierr,n
      ierr = 0
      n = len_real(str)
      read(str(1:n),*,err=9999) r
      ierr = 1
9999  return
      end

c----------------------------------------------------------------------
c     itokenalist
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c --- get character list
c               
c
c --- UPDATES:
c --- INPUT:
c     str     : string to tokenize
c     pos     : start position to start tokenizing 
c     istoken : char used to tokenize 
c
c --- OUTPUT:
c     a()   : text array read
c     returns 0 if no text read
c	  returns the number of texts read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itokenalist(str, pos, istoken, i)
     &   result( n )
     
      integer        :: pos2
      character *(*) :: str
      integer        :: pos
      character *1   :: istoken
      integer        :: n
      character*255   :: i(*)

      n = 0
      DO
         pos2 = INDEX(str(pos:), istoken)
         IF (pos2 .eq. 0) THEN
            n = n + 1
            call zstrcpy(i(n), str(pos:))
c ---           ierr = itoken2i(str(pos:), i(n))
            return 
         END IF
         n = n + 1
         call zstrcpy(i(n), str(pos:pos+pos2-2))
c ---        ierr = itoken2i(str(pos:pos+pos2-2), i(n))
         pos = pos2+pos 
      END DO  
      return
      end function




c----------------------------------------------------------------------
c     itoken
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     get next token
c               
c
c --- UPDATES:
c --- INPUT:
c	  STR:    input string
c     POS:    start position to look in STR
c     ISTOKEN: character to split STR with
c 
c --- OUTPUT:
c     WORD:   string read from STR
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itoken(str, pos, istoken, word)
     &   result( pos2)
      integer        :: pos2
      character *(*) :: str
      character *(*) :: word
      character *255 :: dummy
      integer        :: pos
      character *1   :: istoken

      pos2 = INDEX(str(pos:), istoken)
      IF (pos2 .eq. 0) THEN
         dummy= str(pos:)
         call zstrtrim(dummy)
         call zstrcpy(word,dummy)
         return
      END IF
      dummy= str(pos:pos+pos2-2)
      call zstrtrim(dummy)
      call zstrcpy(word,dummy)
      pos2 = pos2 + pos
      return
      end function




c ----------------------------------------------------------------------
c     itokenptr
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     get next position
c
c --- UPDATES:
c --- INPUT:
c	  STR   : input string
c     POS   : start position in str to start looking
c     ISTOKEN : character for splitting string
c
c --- OUTPUT:
c     RETURNS: start position of ISTOKEN in string
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itokenptr(str, pos, istoken)
      character *(*) :: str
      integer        :: pos
      character (1)  :: istoken
      integer        :: pos2

      pos2 = INDEX(str(pos:), istoken)
      itokenptr = pos2
      return
      end function




c----------------------------------------------------------------------
c     itokenize
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     basic function to look for a parameter in CALPUFF style inputs          
c     of the format
c      comment ! A=1 ! comment
c
c --- UPDATES:
c
c --- INPUT:
c	  STR   : input string
c --- OUTPUT:
c     SPARAM : returns a string containing the parameter name on the leftside "="
c     SVALUE : return a string containing what is on the right side of "="
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function itokenize(str, sparam, svalue)
     &      result(pos)
c --- inputs 
      character *(*) :: str
      character *(*) :: sparam
      character *(*) :: svalue
c --- declarations
      character *255 :: sdummy
      character *255 :: sdummy2
      integer :: pos
      pos = 1
      if( itokenptr(str, pos, '!') .gt. 0) then 
         pos =1
         pos = itoken(str, pos, '!', sdummy)
         pos = itoken(str, pos, '!', sdummy)
         pos =1
         if( itokenptr(sdummy, pos, '=') .gt. 0) then 
            pos =1
            pos = itoken(sdummy, pos, '=', sdummy2)
            call zstrtrim(sdummy2)
            call zstrcpy(sparam,sdummy2)
            pos = itoken(sdummy, pos, '=', sdummy2)
            call zstrtrim(sdummy2)
            call zstrcpy(svalue,sdummy2)
            pos =1   ! return code is good
            return 
         else
            call zstrtrim(sdummy)
            call zstrcpy(sparam,sdummy)
            call zstrcpy(svalue,sdummy)
            pos =1   ! return code is good
            return 
         end if
      end if
      pos = 0
      return 
      end


c----------------------------------------------------------------------
c     LEQ
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     determines if two strings are the same          
c
c --- UPDATES:
c --- INPUT:
c	  STR1:   string 1
c	  STR2:   string 2
c
c --- OUTPUT:
c     returns 0 if string1 equals string2
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function LEQ(str1, str2)
     &   result (ier)
c --- inputs      
      character *(*) :: str1
      character *(*) :: str2
      integer  :: ier
c --- declarations
      integer :: n1
      integer :: n2
      integer :: pos
     
      ier = 0
c --- check string length
      n1 = len_real(str1)
      n2 = len_real(str2)
      if( n1 .ne. n2) goto 9000

c --- remove blanks
c      call zstrtrim(str1)
c      call zstrtrim(str2)

c --- scan strings
      pos = 1
      do while(pos .le. n1)
         if( ichar(str1(pos:pos)) .ne. ichar(str2(pos:pos))) goto 9000
         pos = pos + 1
      end do
      ier = 1
 9000 return         
      end

c without specifying nc
c sparam_in is first token
c then read all tokens=sparam_in until token changes or eof
c input nc is max iv() size
c return nc the number of read as ier
      Function irdparam_avx(io, sparam_in, nc , av)
     &      result(ier)
c ---inputs
      integer :: io
      character *(*) ::sparam_in
      integer  :: nc
      character *(*) :: av(*)
      integer  :: ier
c ---declarations
      integer :: iloop
      integer :: pos
      character *255 :: str
      character *255 :: sparam
      character *255 :: svalue
      integer :: n2 

      n2 = 0
      ier = 0
      rewind (io)
      do while (ier .eq. 0)
         read(io,'(A)',iostat=ierr,err=9000,end=9000) str
         pos = 1
         if (itokenize(str, sparam, sValue) .gt. 0) Then
            if (LEQ(sparam, sparam_in) .gt. 0) Then
               n2 = n2 + 1
               n2 = min(n2, nc)
c               ierr = itoken2i(sValue, ir)
               av(n2) = sValue
            Else
c  raise end flag only when data has been found (n2>0)
c  but now token is not sparam_in, so end of list
               if( n2 .gt. 0 )Then
                  ier = 1
               end if
            end if
         end if
      end do
 9000 if( n2 .gt. 0) Then
         ier = n2
      end if

      return
      end

c----------------------------------------------------------------------
c     irdparam_av
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of texts to read 
c
c --- OUTPUT:
c       av() : array of output text
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_av(io, sparam_in, nc, av)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      character *(*) :: av(*)
      integer  :: ier
c --- declarations
      character*255 :: avread(32)
      integer :: iloop
      integer :: pos
      character *255 :: str
      character *255 :: sparam
      character *255 :: svalue

      ier = 0
      rewind(io)     
      do while (ier .eq. 0)
         read(io,'(A)',iostat=ierr,err=9000,end=9000) str
         pos =1
         if( itokenize( str, sparam, svalue) .gt. 0) then
            if( leq(sparam,sparam_in) .gt.  0 ) then
               if( itokenize( str, sparam, svalue) .gt. 0) then
                  pos = 1
                  n2 = itokenalist(svalue, pos, ',', avread)
                  n2 = min(n2, nc)
                  do iv2=1,n2
                     av(iv2) = avread(iv2)
                  end do
               end if                  
               ier = 1
            end if
         end if
      end do

 9000 return 
      end

c----------------------------------------------------------------------
c     irdparam_av2
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of texts to read 
c
c --- OUTPUT:
c       av(nc,*) : array of output text
c       nr   : number of rows of text 
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_av2(io, sparam_in, nr, nc, av)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      character*(*) :: av(nr,nc)
      integer  :: ier
c --- declarations
      character*255 :: avread(32)
      integer :: iloop
      integer :: pos
      character *255 :: str
      character *255 :: sparam
      character *255 :: svalue
      
      character *255 :: dum

c      ier = 0
c      rewind(io)     
c      do while (ier .eq. 0)
c         read(io,'(A)',iostat=ierr,err=9000,end=9000) str
c         pos =1
c         if( itokenize( str, sparam, svalue) .gt. 0) then
c            if( leq(sparam,sparam_in) .gt. 0 ) then
c               ierr = itoken2i(svalue, nr)
c               inr = 0
c               do while (inr .lt. nr)
c                  read(io,'(A)',iostat=ierr,err=9000,end=9000) str
c                  if( itokenize( str, sparam, svalue) .gt. 0) then
c                     pos = 1
c                     n2 = itokenalist(svalue, pos, ',', avread)
c                     n2 = min(n2, nc)
c                     inr = inr + 1
c                     do iv2=1,n2
c                        dum = avread(iv2)
c                        av(iv2,inr) = avread(iv2)
c                     end do
c                  end if                  
c               end do
c               ier = 1
c            end if
c         end if
c      end do


      ier = 0
      inr = 0
      rewind(io)     
      do while (inr .lt. nr)
         read(io,'(A)',iostat=ierr,err=9000,end=9000) str
         if( itokenize( str, sparam, svalue) .gt. 0) then
            if( leq(sparam,sparam_in) .gt. 0 ) then
               pos = 1
               n2 = itokenalist(svalue, pos, ',', avread)
               n2 = min(n2, nc)
               inr = inr + 1
               do iv2=1,n2
                  dum = avread(iv2)
                  av(inr,iv2) = avread(iv2)
               end do
            end if    
         end if                  
      end do    
      ier = 1

 9000 return 
      end



c without specifying nc
c sparam_in is first token
c then read all tokens=sparam_in until token changes or eof
c input nc is max iv() size
c return nc the number of read as ier
      Function irdparam_ivx(io, sparam_in, nc , iv)
     &      result(ier)
c ---inputs
      integer :: io
      character *(*) ::sparam_in
      integer  :: nc
      integer  :: iv(*)
      integer  :: ier
c ---declarations
      character *255 :: av(nc)

      ier = irdparam_av(io, sparam_in, nc, av)
      if(ier.gt.0) then
         do i=1,ier
            if( len_real(av(i)).gt.0) then
               ierr = itoken2i(av(i), iv(i))
            end if 
         end do
      end if 

      return
      end


c 
c----------------------------------------------------------------------
c     irdparam_iv
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of integer to read 
c
c --- OUTPUT:
c       iv(*) : array of output text
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_iv(io, sparam_in, nc, iv)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      integer  :: iv(*)
      integer  :: ier
c --- declarations
      character *255 :: av(nc)

      ier = 0
      if(irdparam_av(io, sparam_in, nc, av).gt.0) then
         do i=1,nc
            if( len_real(av(i)).gt.0) then
               ierr = itoken2i(av(i), iv(i))
            end if 
            ier = 1
         end do
      end if 
      
      return 
      end


c----------------------------------------------------------------------
c     irdparam_iv2
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of integer to read 
c
c --- OUTPUT:
c       iv(nc,*) : array of output integers
c       nr   : number of rows of integers 
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_iv2(io, sparam_in, nr, nc, iv, nriv, nciv)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      integer  :: iv(nriv, nciv)
      integer  :: ier
c --- declarations
      character *255 :: av(nr,nc)

      ier = 0
      if(irdparam_av2(io, sparam_in, nr, nc, av).gt.0) then
         do i=1,nr
         do j=1,nc
            if( len_real(av(i,j)).gt.0) then
               ierr = itoken2i(av(i,j), iv(i,j))
            end if 
            ier = 1
         end do
         end do
      end if 
      
      return 
      end

c without specifying nc
c sparam_in is first token
c then read all tokens=sparam_in until token changes or eof
c input nc is max iv() size
c return nc the number of read as ier
      Function irdparam_rvx(io, sparam_in, nc , rv)
     &      result(ier)
c ---inputs
      integer :: io
      character *(*) ::sparam_in
      integer  :: nc
      real(8)     :: rv(*)
      integer  :: ier
c ---declarations
      character *255 :: av(nc)

      ier = irdparam_av(io, sparam_in, nc, av)
      if(ier.gt.0) then
         do i=1,ier
            if( len_real(av(i)).gt.0) then
               ierr = itoken2r(av(i), rv(i))
            end if 
         end do
      end if 
      
      return
      end

                                 
c----------------------------------------------------------------------
c     irdparam_rv
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of reals to read 
c
c --- OUTPUT:
c       rv(*) : array of output real
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_rv(io, sparam_in, nc, rv)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      real(8)     :: rv(*)
      integer  :: ier
c --- declarations
      character *255 :: av(nc)

      ier = 0
      if(irdparam_av(io, sparam_in, nc, av).gt.0) then
         do i=1,nc
            if( len_real(av(i)).gt.0) then
               ierr = itoken2r(av(i), rv(i))
            end if 
            ier = 1
         end do
      end if 
      
      return 
      end


c----------------------------------------------------------------------
c     irdparam_rv2
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c       nc : number of reals to read 
c
c --- OUTPUT:
c       rv(nc,*) : array of output real
c       nr   : number of rows of real 
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_rv2(io, sparam_in, nr, nc, rv, nrrv, ncrv)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      real(8)     :: rv(nrrv,ncrv)
      integer  :: ier
c --- declarations
      character *255 :: av(nr,nc)

      ier = 0
      if(irdparam_av2(io, sparam_in, nr, nc, av).gt.0) then
         do i=1,nr
         do j=1,nc
            if( len_real(av(i,j)).gt.0) then
               ierr = itoken2r(av(i,j), rv(i,j))
            end if 
            ier = 1
         end do
         end do
      end if 

      return 
      end


c----------------------------------------------------------------------
c     irdparam_i
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c  
c --- OUTPUT:
c       i :  output integer
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_i(io, sparam_in, i)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      integer  :: i
      integer  :: ier
c --- declarations
      character *255 :: av

      ier = 0
      if(irdparam_a(io, sparam_in, av).gt.0) then
         if( len_real(av).gt.0) then
         ierr = itoken2i(av, i)
         end if 
         ier = 1
      end if 

      return 
      end


c----------------------------------------------------------------------
c     irdparam_r
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c  
c --- OUTPUT:
c       r :  output real
c		function returns 0 if nothing is read
c                        1 if value is read
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_r(io, sparam_in, r)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      real(8)     :: r
      integer  :: ier
c --- declarations
      character *255 :: av

      ier = 0
      if(irdparam_a(io, sparam_in, av).gt.0) then
         if( len_real(av).gt.0) then
         ierr = itoken2r(av, r)
         end if 
         ier = 1
      end if 

      return 
      end

c----------------------------------------------------------------------
c     irdparam_a
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c       io : io UNIT for reading from
c       sparam_in:   parameter name to look for
c  
c --- OUTPUT:
c       a :  output text
c		function returns 0 if nothing is read
c                        1 if value is read
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function irdparam_a(io, sparam_in, a)
     &      result(ier)
c --- inputs 
      integer :: io
      character *(*) ::sparam_in
      character *(*) :: a
      integer  :: ier
c --- declarations
      integer :: iloop
      integer :: pos
      character *255 :: str
      character *255 :: sparam
      character *255 :: svalue

      ier = 0
      rewind(io)     
      do while (ier .eq. 0)
         read(io,'(A)',iostat=ierr,err=9000,end=9000) str
         pos =1
         if( itokenize( str, sparam, svalue) .gt. 0) then
            if( leq(sparam,sparam_in) .gt. 0 ) then
               call zstrcpy(a, svalue)
               ier = 1
            end if
         end if
      end do

 9000 return 
      end

c----------------------------------------------------------------------
c     
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES:
c --- INPUT:
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      function len_real(str)
     &   result(n)
      integer :: n,ic
      character*(*) :: str
      integer :: idone
      n = len(str)
      
      idone = 0
      do while (idone .eq.0) 
          idone =1    
          if(ichar(str(n:n)) .eq. 32 
     &  .or. ichar(str(n:n)) .eq. 9  
     &  .or. ichar(str(n:n)) .eq. 0) then
              idone=0
              n = n-1
              if(n.lt.1) idone=1
          end if               
      end do 
c      ic = ichar(str(n:n) )
      return
      end
      
      
      
      
c----------------------------------------------------------------------
c     zstrcpy
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c copies str1 to str2      
c               
c
c --- UPDATES:
c --- INPUT:
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      subroutine zstrcpy(str2,str1)
      character*(*) :: str2
      character*(*) :: str1
      integer :: n1
      integer :: n2
      integer :: i
      n1 = len_real(str1)
      n2 = len(str2)
      n1 = min(n1,n2)
      do i=1,n1
         str2(i:i) = str1(i:i)
      end do
      if(n1+1 .le. n2) then 
         do i=n1+1,n2
            str2(i:i) = char(0)
         end do
      end if 
      return
      end
      
      
      
c----------------------------------------------------------------------
c     zstrtrim
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c     removes blanks from left           
c
c --- UPDATES:
c --- INPUT:
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
c     
      subroutine zstrtrim(str)
      character *(*) :: str
      integer :: n1
      integer :: n2
      integer :: i,pos
      n1 = len_real(str)
      n2 = len(str)
     
      pos = 1
      do while((ichar(str(pos:pos)) .eq.32  
     & .or. ichar(str(pos:pos)) .eq.9)  .and. pos.lt.n1)
         pos = pos + 1
      end do
      
      if( pos .le. n1) then
         do i=1,n1-pos+1
            str(i:i) = str(pos+i-1:pos+i-1)
         end do
      end if
      
      if(i.gt.0 .and. i .le. n2) then 
         do i=i,n2
            str(i:i) = char(0)
         end do
      end if 
      return 
      end 